


###########################################################################################
### Title: Toward computerized efficient estimation in infinite-dimensional models
### The code below was used to produce the plots for Example 1 (i.e., Figures 1, 2 and 3).
### Example 1: Average value of density function with known mean
###########################################################################################

### Helper function

integratepwise<-function(f,a,b,c,d){

	integrate(f,a,b,rel.tol=1e-13)$value+
	integrate(f,b,c,rel.tol=1e-13)$value+
	integrate(f,c,d,rel.tol=1e-13)$value;
	
}

### Setting parameter of the distribution at which EIF is evaluated

buffer = 0.0001
alpha<-3; beta<-5;
meanbeta<-alpha/(alpha+beta);
p0<-function(u) dbeta(u,alpha,beta);

### Projecting onto the model

perturbeddensity<-function(eps,x,lambda){
	
	dens<-function(u) (1-eps)*p0(u)+eps*dunif(u-x,min=-lambda,max=lambda);
	dens<-Vectorize(dens);
	
}

projectedp<-function(q,mu,b=mu,c=mu){

	constraint<-function(u){
		integrand<-function(x) (x/(1-u*(x-mu))-mu)*q(x);
		return(integratepwise(integrand,minx,b,c,maxx));
	}
	constraint<-Vectorize(constraint);
	
	xi<-uniroot(constraint,lower=1/(minx-mu)+buffer,upper=1/(maxx-mu)-buffer,tol=1e-20,maxiter=10000)$root;	
	
	d<-function(x) 1-xi*(x-mu);
	ptilde<-function(x) q(x)/d(x);
	return(ptilde);
	
}

### Analytic forms of the nonparametric and semiparametric EIFs

trueEIF.np<-function(u) 2*(p0(u)-computeparameter());

trueEIF.knownmean<-function(u,mu){
	psi0<-computeparameter();	
	int0<-integrate(function(x) x*p0(x)^2,minx,maxx)$value;
	var0<-integrate(function(x) (x-mu)^2*p0(x),minx,maxx)$value;
	2*(p0(u)-psi0)-2*(int0-mu*psi0)*(u-mu)/var0;
}

### Computing the parameter and the perturbed parameter

computeparameter<-function() integrate(function(u) p0(u)^2,minx,maxx)$value;

computeperturbedparameter<-function(eps,x,lambda){

	p0eps<-perturbeddensity(eps,x,lambda);
	p0.tilde<-projectedp(p0eps,mu,x-lambda,x+lambda);
	return(integratepwise(function(u) p0.tilde(u)^2,minx,x-lambda,x+lambda,maxx));
	
}


### Computing EIF via projection of perturbed distribution using two-point scheme

eifbyperturbation<-function(eps,x,lambda) (computeperturbedparameter(eps,x,lambda)-computeparameter())/eps;

### Numerical run, as presented in manuscript

minx=0; maxx=1; mu=meanbeta;
x=0.6;
epsvec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)
lambdavec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)

e<-NULL; l<-NULL; v<-NULL; sv<-NULL;
for(i in 1:10){
	for(j in 1:10){
		k<-(i-1)*10+j;
		e[k]<-i; l[k]<-j-1;
		v[k]<-eifbyperturbation(epsvec[i],x,lambdavec[j])
	}
}
truth<-trueEIF.knownmean(x,mu)
svorig<-round(signif(abs((v-truth)/truth)*100,2),1)
sv<-apply(matrix(svorig,nrow=1),2,function(u) min(u,100))

### Producing Figure 1 from output above

vp<-NULL
for(i in 1:length(sv)){

	if(abs(v[i])<10){
		vp[i]=format(signif(v[i],3),nsmall=3);
	}
	if(abs(v[i])>=10&abs(v[i])<100){
		vp[i]=format(signif(v[i],3),nsmall=2);
	}
	if(abs(v[i])>=100&abs(v[i]<1000)){
		vp[i]=format(signif(v[i],3),nsmall=1);
	}
	if(v[i]>=1000){ vp[i]=">999" }
}

par(mar=c(5,6,1,0)+0.1)
plot(5,5,type='n',xlim=c(-0.5,10),ylim=c(0,11),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
text(x=l,y=e,labels=vp,col=1,cex=0.78)
labx=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
laby=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
axis(1,at=0:9,labels=labx,xlab=2)
axis(2,at=1:10,labels=laby,las=1)
title(xlab=expression(lambda),cex.lab=1.5)
mtext(expression(epsilon),side=2,cex=1.7,line=4)

### Producing Figure 2 from output above

c<-NULL
for(i in 1:length(sv)){

	if(sv[i]==0) c[i]="forestgreen"
	if(sv[i]>0&&sv[i]<=1) c[i]="greenyellow"
	if(sv[i]>1&&sv[i]<=10) c[i]="gold"
	if(sv[i]>10&&sv[i]<=20) c[i]="orange1"
	if(sv[i]>20) c[i]="red"
	
}

par(mar=c(5,6,1,0)+0.1)
plot(5,5,type='n',xlim=c(-0.5,13),ylim=c(0,11),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
points(x=l,y=e,pch=22,cex=3)
points(x=l,y=e,pch=15,col=c,cex=2.7)
labx=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
laby=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
axis(1,at=0:9,labels=labx,xlab=2)
axis(2,at=1:10,labels=laby,las=1)
title(xlab=expression(paste(lambda,"                    ")),cex.lab=1.5)
mtext(expression(epsilon),side=2,cex=1.7,line=4)
legend(10,7.5,legend=c("<0.1%","0.1-1%","1-10%","10-100%",">100%"),fill=c("forestgreen","greenyellow","gold","orange1","red"),bty="n")


### Computing EIF via projection of perturbed distribution using three-point scheme

eifbyperturbation<-function(eps,x,lambda) (-0.5*computeperturbedparameter(2*eps,x,lambda)+2*computeperturbedparameter(eps,x,lambda)-1.5*computeparameter())/eps;

### Numerical run

minx=0; maxx=1; mu=meanbeta;
x=0.6;
epsvec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)
lambdavec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)

e<-NULL; l<-NULL; v<-NULL; sv<-NULL;
for(i in 1:10){
	for(j in 1:10){
		k<-(i-1)*10+j;
		e[k]<-i; l[k]<-j-1;
		v[k]<-eifbyperturbation(epsvec[i],x,lambdavec[j])
	}
}
truth<-trueEIF.knownmean(x,mu)
svorig<-round(signif(abs((v-truth)/truth)*100,2),1)
sv<-apply(matrix(svorig,nrow=1),2,function(u) min(u,100))

### Producing upper-right exhibit of Figure 3 from output above

c<-NULL
for(i in 1:length(sv)){

	if(sv[i]==0) c[i]="forestgreen"
	if(sv[i]>0&&sv[i]<=1) c[i]="greenyellow"
	if(sv[i]>1&&sv[i]<=10) c[i]="gold"
	if(sv[i]>10&&sv[i]<=20) c[i]="orange1"
	if(sv[i]>20) c[i]="red"
	
}

par(mar=c(5,6,1,0)+0.1)
plot(5,5,type='n',xlim=c(-0.5,13),ylim=c(0,11),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
points(x=l,y=e,pch=22,cex=3)
points(x=l,y=e,pch=15,col=c,cex=2.7)
labx=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
laby=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
axis(1,at=0:9,labels=labx,xlab=2)
axis(2,at=1:10,labels=laby,las=1)
title(xlab=expression(paste(lambda,"                    ")),cex.lab=1.5)
mtext(expression(epsilon),side=2,cex=1.7,line=4)
legend(10,7.5,legend=c("<0.1%","0.1-1%","1-10%","10-100%",">100%"),fill=c("forestgreen","greenyellow","gold","orange1","red"),bty="n")


### Computing EIF via projection of perturbed distribution using four-point scheme

eifbyperturbation<-function(eps,x,lambda) ((1/3)*computeperturbedparameter(3*eps,x,lambda)-(3/2)*computeperturbedparameter(2*eps,x,lambda)+3*computeperturbedparameter(eps,x,lambda)-(11/6)*computeparameter())/eps;

### Numerical run, as presented in submitted manuscript

minx=0; maxx=1; mu=meanbeta;
x=0.6;
epsvec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)
lambdavec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)

e<-NULL; l<-NULL; v<-NULL; sv<-NULL;
for(i in 1:10){
	for(j in 1:10){
		k<-(i-1)*10+j;
		e[k]<-i; l[k]<-j-1;
		v[k]<-eifbyperturbation(epsvec[i],x,lambdavec[j])
	}
}
truth<-trueEIF.knownmean(x,mu)
svorig<-round(signif(abs((v-truth)/truth)*100,2),1)
sv<-apply(matrix(svorig,nrow=1),2,function(u) min(u,100))

### Producing lower-left exhibit of Figure 3 from output above

c<-NULL
for(i in 1:length(sv)){

	if(sv[i]==0) c[i]="forestgreen"
	if(sv[i]>0&&sv[i]<=1) c[i]="greenyellow"
	if(sv[i]>1&&sv[i]<=10) c[i]="gold"
	if(sv[i]>10&&sv[i]<=20) c[i]="orange1"
	if(sv[i]>20) c[i]="red"
	
}

par(mar=c(5,6,1,0)+0.1)
plot(5,5,type='n',xlim=c(-0.5,13),ylim=c(0,11),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
points(x=l,y=e,pch=22,cex=3)
points(x=l,y=e,pch=15,col=c,cex=2.7)
labx=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
laby=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
axis(1,at=0:9,labels=labx,xlab=2)
axis(2,at=1:10,labels=laby,las=1)
title(xlab=expression(paste(lambda,"                    ")),cex.lab=1.5)
mtext(expression(epsilon),side=2,cex=1.7,line=4)
legend(10,7.5,legend=c("<0.1%","0.1-1%","1-10%","10-100%",">100%"),fill=c("forestgreen","greenyellow","gold","orange1","red"),bty="n")


### Computing EIF via projection of perturbed distribution using five-point scheme

eifbyperturbation<-function(eps,x,lambda) ((-1/4)*computeperturbedparameter(4*eps,x,lambda)+(4/3)*computeperturbedparameter(3*eps,x,lambda)+(-3)*computeperturbedparameter(2*eps,x,lambda)+4*computeperturbedparameter(eps,x,lambda)+(-25/12)*computeparameter())/eps;

### Numerical run, as presented in submitted manuscript

minx=0; maxx=1; mu=meanbeta;
x=0.6;
epsvec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)
lambdavec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)

e<-NULL; l<-NULL; v<-NULL; sv<-NULL;
for(i in 1:10){
	for(j in 1:10){
		k<-(i-1)*10+j;
		e[k]<-i; l[k]<-j-1;
		v[k]<-eifbyperturbation(epsvec[i],x,lambdavec[j])
	}
}
truth<-trueEIF.knownmean(x,mu)
svorig<-round(signif(abs((v-truth)/truth)*100,2),1)
sv<-apply(matrix(svorig,nrow=1),2,function(u) min(u,100))

### Producing lower-right exhibit of Figure 3 from output above

c<-NULL
for(i in 1:length(sv)){

	if(sv[i]==0) c[i]="forestgreen"
	if(sv[i]>0&&sv[i]<=1) c[i]="greenyellow"
	if(sv[i]>1&&sv[i]<=10) c[i]="gold"
	if(sv[i]>10&&sv[i]<=20) c[i]="orange1"
	if(sv[i]>20) c[i]="red"
	
}

par(mar=c(5,6,1,0)+0.1)
plot(5,5,type='n',xlim=c(-0.5,13),ylim=c(0,11),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
points(x=l,y=e,pch=22,cex=3)
points(x=l,y=e,pch=15,col=c,cex=2.7)
labx=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
laby=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
axis(1,at=0:9,labels=labx,xlab=2)
axis(2,at=1:10,labels=laby,las=1)
title(xlab=expression(paste(lambda,"                    ")),cex.lab=1.5)
mtext(expression(epsilon),side=2,cex=1.7,line=4)
legend(10,7.5,legend=c("<0.1%","0.1-1%","1-10%","10-100%",">100%"),fill=c("forestgreen","greenyellow","gold","orange1","red"),bty="n")


###########################################################################################
### Title: Toward computerized efficient estimation in infinite-dimensional models
### The code below was used to produce the plots for Example 2 (i.e., Figure 4).
### Example 2: Center of a symmetric distribution
###########################################################################################

### Helper function

integratepwise<-function(f,a,edge){

	edge = abs(edge)

	integrate(f,-edge,a[1],abs.tol=1e-11)$value+
	integrate(f,a[1],a[2],abs.tol=1e-11)$value+
	integrate(f,a[2],a[3],abs.tol=1e-11)$value+
	integrate(f,a[3],a[4],abs.tol=1e-11)$value+
	integrate(f,a[4],edge,abs.tol=1e-11)$value;
	
}

### Setting parameter of the distribution at which EIF is evaluated

mu0<-0
p = function(u) dnorm(u,mean=mu0,sd=1)
p_prime = function(u) -(u-mu0)*exp(-(u-mu0)^2/2)/sqrt(2*pi)

### Projecting onto the model

perturbeddensity<-function(p,eps,x,lambda){
	
	dens<-function(u) (1-eps)*p(u)+eps*dunif(u-x,min=-lambda,max=lambda);
	dens<-Vectorize(dens);
	
}

projectedp_score_KL<-function(p,eps,x,lambda,minmu,maxmu){

	q = perturbeddensity(p,eps,x,lambda);

	score<-function(mu){
		integrand<-function(u) ifelse(q(u)<1e-15,0,eps*p_prime(2*mu-u)*q(u)/(q(u)+q(2*mu-u)));
		midpoints<-sort(c(x-lambda,x+lambda,2*mu-x-lambda,2*mu-x+lambda));
		return(integratepwise(integrand,a=midpoints,edge=8));
	}
	score<-Vectorize(score);
	
	mutilde<-uniroot(score,lower=minmu,upper=maxmu,tol=1e-20)$root;	
	return(mutilde);
	
}

projectedp_score_H<-function(p,eps,x,lambda,minmu,maxmu){

	q = perturbeddensity(p,eps,x,lambda);

	score<-function(mu){
		integrand<-function(u) ifelse(q(u)<1e-15,0,eps*p_prime(2*mu-u)*sqrt(q(u)/q(2*mu-u)));
		midpoints<-sort(c(x-lambda,x+lambda,2*mu-x-lambda,2*mu-x+lambda));
		return(integratepwise(integrand,a=midpoints,edge=8));
	}
	score<-Vectorize(score);
	
	mutilde<-uniroot(score,lower=minmu,upper=maxmu,tol=1e-20)$root;	
	return(mutilde);
	
}

### Analytic form of EIF

trueEIF<-function(u) return(u-mu0);

### Computing the perturbed parameter

computeperturbedparameter_KL<-function(eps,x,lambda){

	mu.tilde<-projectedp_score_KL(p,eps,x,lambda,minmu=-0.1,maxmu=0.1);
	return(mu.tilde);	

}

computeperturbedparameter_H<-function(eps,x,lambda){

	mu.tilde<-projectedp_score_H(p,eps,x,lambda,minmu=-0.1,maxmu=0.1);
	return(mu.tilde);	

}


### Computing EIF via projection of perturbed distribution using two-point scheme

eifbyperturbation_KL<-function(eps,x,lambda) (computeperturbedparameter_KL(eps,x,lambda)-mu0)/eps;
eifbyperturbation_H<-function(eps,x,lambda) (computeperturbedparameter_H(eps,x,lambda)-mu0)/eps;

### Numerical run for Kullback-Leibler implementation, as presented in manuscript

x=1;
epsvec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)
lambdavec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)

e<-NULL; l<-NULL; v<-NULL; sv<-NULL;
for(i in 1:10){
	for(j in 1:10){
		k<-(i-1)*10+j;
		e[k]<-i; l[k]<-j-1;
		v[k]<-eifbyperturbation_KL(epsvec[i],x,lambdavec[j])
	}
}
truth<-trueEIF(x)
svorig<-round(signif(abs((v-truth)/truth)*100,2),1)
sv<-apply(matrix(svorig,nrow=1),2,function(u) min(u,100))

### Producing left exhibit of Figure 4 from output above

vp<-NULL
for(i in 1:length(sv)){

	if(abs(v[i])<10){
		vp[i]=format(signif(v[i],3),nsmall=3);
	}
	if(abs(v[i])>=10&abs(v[i])<100){
		vp[i]=format(signif(v[i],3),nsmall=2);
	}
	if(abs(v[i])>=100&abs(v[i]<1000)){
		vp[i]=format(signif(v[i],3),nsmall=1);
	}
	if(v[i]>=1000){ vp[i]=">999" }
}

c<-NULL
for(i in 1:length(sv)){

	if(sv[i]==0) c[i]="forestgreen"
	if(sv[i]>0&&sv[i]<=1) c[i]="greenyellow"
	if(sv[i]>1&&sv[i]<=10) c[i]="gold"
	if(sv[i]>10&&sv[i]<=20) c[i]="orange1"
	if(sv[i]>20) c[i]="red"
	
}

par(mar=c(5,6,1,0)+0.1)
plot(5,5,type='n',xlim=c(-0.5,13),ylim=c(0,11),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
points(x=l,y=e,pch=22,cex=3)
points(x=l,y=e,pch=15,col=c,cex=2.7)
labx=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
laby=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
axis(1,at=0:9,labels=labx,xlab=2)
axis(2,at=1:10,labels=laby,las=1)
title(xlab=expression(paste(lambda,"                    ")),cex.lab=1.5)
mtext(expression(epsilon),side=2,cex=1.7,line=4)
legend(10,7.5,legend=c("<0.1%","0.1-1%","1-10%","10-100%",">100%"),fill=c("forestgreen","greenyellow","gold","orange1","red"),bty="n")

### Numerical run for Hellinger implementation (not presented in manuscript)

x=1;
epsvec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)
lambdavec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)

e<-NULL; l<-NULL; v<-NULL; sv<-NULL;
for(i in 1:10){
	for(j in 1:10){
		k<-(i-1)*10+j;
		e[k]<-i; l[k]<-j-1;
		v[k]<-eifbyperturbation_H(epsvec[i],x,lambdavec[j])
	}
}
truth<-trueEIF(x)
svorig<-round(signif(abs((v-truth)/truth)*100,2),1)
sv<-apply(matrix(svorig,nrow=1),2,function(u) min(u,100))

### Producing right exhibit of Figure 4 from output above

vp<-NULL
for(i in 1:length(sv)){

	if(abs(v[i])<10){
		vp[i]=format(signif(v[i],3),nsmall=3);
	}
	if(abs(v[i])>=10&abs(v[i])<100){
		vp[i]=format(signif(v[i],3),nsmall=2);
	}
	if(abs(v[i])>=100&abs(v[i]<1000)){
		vp[i]=format(signif(v[i],3),nsmall=1);
	}
	if(v[i]>=1000){ vp[i]=">999" }
}

c<-NULL
for(i in 1:length(sv)){

	if(sv[i]==0) c[i]="forestgreen"
	if(sv[i]>0&&sv[i]<=1) c[i]="greenyellow"
	if(sv[i]>1&&sv[i]<=10) c[i]="gold"
	if(sv[i]>10&&sv[i]<=20) c[i]="orange1"
	if(sv[i]>20) c[i]="red"
	
}

par(mar=c(5,6,1,0)+0.1)
plot(5,5,type='n',xlim=c(-0.5,13),ylim=c(0,11),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
points(x=l,y=e,pch=22,cex=3)
points(x=l,y=e,pch=15,col=c,cex=2.7)
labx=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
laby=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
axis(1,at=0:9,labels=labx,xlab=2)
axis(2,at=1:10,labels=laby,las=1)
title(xlab=expression(paste(lambda,"                    ")),cex.lab=1.5)
mtext(expression(epsilon),side=2,cex=1.7,line=4)
legend(10,7.5,legend=c("0%","0-1%","1-10%","10-100%",">100%"),fill=c("forestgreen","greenyellow","gold","orange1","red"),bty="n")


###########################################################################################
### Title: Toward computerized efficient estimation in infinite-dimensional models
### The code below was used to produce the plots for Example 3 (i.e., Figure 5).
### Example 3: Coefficient in semiparametric Poisson regression model.
###########################################################################################

### Helper functions

expit<-plogis;
logit<-qlogis;

integratepwise<-function(f,a,b,c,d){

	integrate(f,a,b,rel.tol=1e-13)$value+
	integrate(f,b,c,rel.tol=1e-13)$value+
	integrate(f,c,d,rel.tol=1e-13)$value;
	
}

### Setting parameters of the distribution at which EIF is evaluated

beta_0 = log(2);
g_0 = function(w1) w1;
pi_0 = function(w1) expit(1-2*w1);

mean_ZY = exp(beta_0)*integrate(function(w1) pi_0(w1)*exp(g_0(w1)),lower=0,upper=1,abs.tol=1e-20)$val;
mean_YgivenW = function(w1) exp(g_0(w1))*(1+pi_0(w1)*(exp(beta_0)-1));

R_0 = function(w1) pi_0(w1)*exp(beta_0)/(pi_0(w1)*exp(beta_0)+1-pi_0(w1))
EIFdenom_integrand = function(w1) exp(g_0(w1))*R_0(w1)^2*(1-pi_0(w1))+exp(beta_0+g_0(w1))*(1-R_0(w1))^2*pi_0(w1)
EIFdenom = integrate(EIFdenom_integrand,0,1,abs.tol=1e-20)$value

### Analytic form of EIF

trueEIF<-function(x){
	
	y=x[1]; z=x[2]; w=x[3];
	return((z-R_0(w))*(y-exp(beta_0*z+g_0(w)))/EIFdenom);
	
}

### Computing the perturbed parameter

computeperturbedparameter<-function(eps,x,lambda){

	y=x[1]; z=x[2]; w=x[3];

	estfunc = function(beta){
		
		integrand = function(w1){
			
			A = (1-eps)*exp(beta)*pi_0(w1)*dunif(w1)+eps*z*exp(beta*z)*dunif(w1-w,min=-lambda,max=lambda);
			B = (1-eps)*(1+pi_0(w1)*(exp(beta)-1))*dunif(w1)+eps*exp(beta*z)*dunif(w1-w,min=-lambda,max=lambda);
			C = (1-eps)*mean_YgivenW(w1)*dunif(w1)+eps*y*dunif(w1-w,min=-lambda,max=lambda);
			
			return(A/B*C)
			
		}
		
		lowerlim = min(0,w-lambda);
		upperlim = max(1,w+lambda);
		
		funcval = (1-eps)*mean_ZY+eps*z*y-integratepwise(integrand,lowerlim,w-lambda,w+lambda,upperlim);
		return(funcval)
		
	}
	estfunc = Vectorize(estfunc)
	
	return(uniroot(estfunc,lower=beta_0-1,upper=beta_0+1,tol=1e-15,maxiter=10000)$root);

}


### Computing EIF via projection of perturbed distribution using two-point scheme

eifbyperturbation<-function(eps,x,lambda) (computeperturbedparameter(eps,x,lambda)-beta_0)/eps;

### Numerical run, as presented in manuscript

x=c(1,0,0.7);
epsvec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)
lambdavec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)

e<-NULL; l<-NULL; v<-NULL; sv<-NULL;
for(i in 1:10){
	for(j in 1:10){
		k<-(i-1)*10+j;
		e[k]<-i; l[k]<-j-1;
		v[k]<-eifbyperturbation(epsvec[i],x,lambdavec[j])
	}
}

vp<-NULL
for(i in 1:length(v)){

	if(abs(v[i])<10){
		vp[i]=format(signif(v[i],3),nsmall=3);
	}
	if(abs(v[i])>=10&abs(v[i])<100){
		vp[i]=format(signif(v[i],3),nsmall=2);
	}
	if(abs(v[i])>=100&abs(v[i]<1000)){
		vp[i]=format(signif(v[i],3),nsmall=1);
	}
	if(v[i]>=1000){ vp[i]=">999" }
}

truth<-trueEIF(x)
svorig<-round(signif(abs((v-truth)/truth)*100,2),1)
sv<-apply(matrix(svorig,nrow=1),2,function(u) min(u,100))

### Producing Figure 5 from output above

c<-NULL
for(i in 1:length(sv)){

	if(sv[i]==0) c[i]="forestgreen"
	if(sv[i]>0&&sv[i]<=1) c[i]="greenyellow"
	if(sv[i]>1&&sv[i]<=10) c[i]="gold"
	if(sv[i]>10&&sv[i]<=20) c[i]="orange1"
	if(sv[i]>20) c[i]="red"
	
}

par(mar=c(5,6,1,0)+0.1)
plot(5,5,type='n',xlim=c(-0.5,13),ylim=c(0,11),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
points(x=l,y=e,pch=22,cex=3)
points(x=l,y=e,pch=15,col=c,cex=2.7)
labx=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
laby=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
axis(1,at=0:9,labels=labx,xlab=2)
axis(2,at=1:10,labels=laby,las=1)
title(xlab=expression(paste(lambda,"                    ")),cex.lab=1.5)
mtext(expression(epsilon),side=2,cex=1.7,line=4)
legend(10,7.5,legend=c("<0.1%","0.1-1%","1-10%","10-100%",">100%"),fill=c("forestgreen","greenyellow","gold","orange1","red"),bty="n")



###########################################################################################
### Title: Toward computerized efficient estimation in infinite-dimensional models
### The code below was used to produce the plots for Example 4 (i.e., Figures 6 and 7).
### Example 4: G-computation parameter under Markovian structure
###########################################################################################

### Helper functions

expit<-plogis;
logit<-qlogis;

### Setting parameters of the distribution at which EIF is evaluated

clipval<-10;
L1fun<-function(u) u*as.numeric(abs(u)<clipval)+clipval*sign(u)*as.numeric(abs(u)>=clipval);

meanA0<-function(l0) expit(-1+0.5*l0);
meanL1<-function(a0,l0) (3*l0-3*a0);
meanA1<-function(l1,a0,l0) expit(-5+L1fun(l1)+a0+0.5*l0);
meanY<-function(a1,l1,a0) expit(-1+0.5*L1fun(l1)-0.5*a1-a0);

### Projecting onto model

perturbedcond.Y<-function(eps,o,lambda){

	l0support<-(0:4);
	nsupport<-length(l0support);

	if(o[4]==1&&o[2]==1){
	
		condprob<-function(l1){
			num<-(1-eps)*sum(meanY(1,l1,1)*meanA1(l1,1,l0support)*dnorm(l1,mean=meanL1(1,l0support),sd=2)*
					meanA0(l0support)*(1/5))+eps*dunif(l1-o[3],min=-lambda,max=lambda)*as.numeric(o[5]==1);
			denom<-(1-eps)*sum(meanA1(l1,1,l0support)*dnorm(l1,mean=meanL1(1,l0support),sd=2)*
							meanA0(l0support)*(1/5))+eps*dunif(l1-o[3],min=-lambda,max=lambda);
			return(num/denom);
		}
		condprob<-Vectorize(condprob);
	
	}
	
	if(o[4]!=1||o[2]!=1){
		
		condprob<-function(l1) meanY(1,l1,1);
		condprob<-Vectorize(condprob);
		
	}
	
	return(condprob);
	
}

perturbedcond.L1<-function(l0,eps,o,lambda){
	
	if(o[2]==1&&o[1]==l0){
	
		conddens<-function(l1){
			num<-(1-eps)*dnorm(l1,mean=meanL1(1,l0),sd=2)*meanA0(l0)*(1/5)+
					eps*dunif(l1-o[3],min=-lambda,max=lambda);
			denom<-(1-eps)*meanA0(l0)*(1/5)+eps;
			return(num/denom);
		}
		conddens<-Vectorize(conddens);
	
	}
	
	if(o[2]!=1||o[1]!=l0){
		
		conddens<-function(l1) dnorm(l1,mean=meanL1(1,l0),sd=2);
		conddens<-Vectorize(conddens);
		
	}
	
	return(conddens);
	
}

perturbedmarg.L0<-function(eps,o){
	
	margpdf<-function(l0){
		return((1-eps)*(1/5)*as.numeric(l0%in%(0:4))+eps*as.numeric(l0==o[1]));
	}
	margpdf<-Vectorize(margpdf);
	
	return(margpdf);

}

### Analytic form of semiparametric EIF

trueEIF.markov<-function(o){
	
	l0support<-(0:4);
	nsupport<-length(l0support);
	
	l0<-o[1]; a0<-o[2]; l1<-o[3]; a1<-o[4]; y<-o[5];
	
	conditional<-function(u){
		(y*meanY(1,l1,1)+(1-y)*(1-meanY(1,l1,1)))*meanA1(l1,1,u)*
				dnorm(l1,mean=meanL1(1,u),sd=2)*meanA0(u)*(1/nsupport);
	}
	conditional<-Vectorize(conditional);
	normconstant<-sum(conditional(l0support));
	conditionalnorm<-function(u) conditional(u)/normconstant;
	
	T2fun<-function(l0val) (1/meanA0(l0val))*(1/meanA1(l1,1,l0val))*conditionalnorm(l0val);
	T2fun<-Vectorize(T2fun);
	T2value<-sum(T2fun(l0support));
	
	T1value<-1/meanA0(l0);
	
	Q1<-meanY(1,l1,1);
	Q0<-integrate(function(u) meanY(1,u,1)*dnorm(u,mean=meanL1(1,l0),sd=2),-Inf,Inf,rel.tol=1e-10)$value;
	
	DM2<-a1*a0*T2value*(y-Q1);
	DM1<-a0*T1value*(Q1-Q0);
	DM0<-Q0-computeparameter();
	
	return(DM0+DM1+DM2);
	
}

### Computing the parameter and the perturbed parameter

computeparameter<-function(){

	intvec<-NULL;
	l0support<-(0:4);
	nsupport<-length(l0support);
	
	for(i in 1:nsupport){
		
		integrand<-function(u) meanY(1,u,1)*dnorm(u,mean=meanL1(1,l0support[i]),sd=2);
		integrand<-Vectorize(integrand);
		intvec[i]<-integrate(integrand,-70,+70,rel.tol=1e-13)$value;
		
	}
	
	return(mean(intvec));
		
}

computeperturbedparameter<-function(eps,o,lambda){
	
	intvec<-NULL;
	l0support<-(0:4);
	nsupport<-length(l0support);

	mu.tilde<-perturbedcond.Y(eps,o,lambda);
	p0.tilde<-perturbedmarg.L0(eps,o);
	
	for(i in 1:nsupport){
		
		p1.tilde<-perturbedcond.L1(l0support[i],eps,o,lambda);
		
		integrand<-function(u) mu.tilde(u)*p1.tilde(u);
		integrand<-Vectorize(integrand);
		intvec[i]<-(integrate(integrand,-70,o[3]-lambda,rel.tol=1e-13)$value+
						integrate(integrand,o[3]-lambda,o[3]+lambda,rel.tol=1e-13)$value+
						integrate(integrand,o[3]+lambda,70,rel.tol=1e-13)$value)*
						p0.tilde(l0support[i]);
		
	}
	
	return(sum(intvec));	

}


### Computing EIF via projection of perturbed distribution using two-point scheme

eifbyperturbation<-function(eps,o,lambda) (computeperturbedparameter(eps,o,lambda)-computeparameter())/eps;

### Computing EIF via projection of perturbed distribution using analytic derivative

computederivative<-function(lambda,o){

	l0<-o[1]; a0<-o[2]; l1<-o[3]; a1<-o[4]; y<-o[5];

	l0support<-(0:4);
	nsupport<-length(l0support);
	
	denomint1<-function(u){
		temp<-0;
		for(i in 1:nsupport){
			temp<-temp+meanA1(u,1,l0support[i])*
					   dnorm(u,mean=meanL1(1,l0support[i]),sd=2)*
					   meanA0(l0support[i])*(1/5);	
		}
		return(temp);
	}

	int1<-0;
	if(a0*a1==1){
	
		integrand<-function(u){
		
			temp<-0;
			for(i in 1:nsupport) temp<-temp+dnorm(u,mean=meanL1(1,l0support[i]),sd=2)*(1/5);
			val<-dunif(u-l1,min=-lambda,max=lambda)*(y-meanY(1,u,1))*temp/denomint1(u);
			
		}
		integrand<-Vectorize(integrand);
		
		int1<-integrate(integrand,-70,l1-lambda,rel.tol=1e-13)$value+
				integrate(integrand,l1-lambda,l1+lambda,rel.tol=1e-13)$value+
				integrate(integrand,l1+lambda,70,rel.tol=1e-13)$value;
		
	}
	
	if(a0==1){
		
		integrand<-function(u){
		
			meanY(1,u,1)*(dunif(u-l1,min=-lambda,max=lambda)-dnorm(u,mean=meanL1(1,l0),sd=2))/meanA0(l0);	
			
		}
		integrand<-Vectorize(integrand);
		
		int2<-integrate(integrand,-70,l1-lambda,rel.tol=1e-13)$value+
				integrate(integrand,l1-lambda,l1+lambda,rel.tol=1e-13)$value+
				integrate(integrand,l1+lambda,70,rel.tol=1e-13)$value;
		
	}
	
	integrand<-function(u) meanY(1,u,1)*dnorm(u,mean=meanL1(1,l0),sd=2);
	int3<-integrate(integrand,-70,70,rel.tol=1e-13)$value-computeparameter();
	
	return(int1+int2+int3);
	
}


### Numerical run based on two-point scheme, as presented in manuscript

o<-c(2,1,3,1,1)
epsvec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)
lambdavec<-c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10)

e<-NULL; l<-NULL; v<-NULL; sv<-NULL;
for(i in 1:10){
	for(j in 1:10){
		k<-(i-1)*10+j;
		e[k]<-i; l[k]<-j-1;
		v[k]<-eifbyperturbation(epsvec[i],o,lambdavec[j])
	}
}
truth<-trueEIF.markov(o)
sv<-round(signif(abs(v-truth)/truth*100,2),1)

### Producing Figure 6 from output above

c<-NULL
for(i in 1:length(sv)){

	if(sv[i]==0) c[i]="forestgreen"
	if(sv[i]>0&&sv[i]<=1) c[i]="greenyellow"
	if(sv[i]>1&&sv[i]<=10) c[i]="gold"
	if(sv[i]>10&&sv[i]<=30) c[i]="orange1"
	if(sv[i]>30) c[i]="red"
	
}

par(mar=c(5,6,1,0)+0.1)
plot(5,5,type='n',xlim=c(-0.5,13),ylim=c(0,11),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
points(x=l,y=e,pch=22,cex=3)
points(x=l,y=e,pch=15,col=c,cex=2.7)
labx=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
laby=parse(text=paste(10, "^ ",seq(-1,-10,by=-1),sep=""))
axis(1,at=0:9,labels=labx,xlab=2)
axis(2,at=1:10,labels=laby,las=1)
title(xlab=expression(paste(lambda,"                    ")),cex.lab=1.5)
mtext(expression(epsilon),side=2,cex=1.7,line=4)
legend(10.5,7.5,legend=c("<0.1%","0.1-1%","1-10%","10-30%",">30%"),fill=c("forestgreen","greenyellow","gold","orange1","red"),bty="n")


### Numerical run based on analytic derivative, as presented in manuscript

o<-c(2,1,3,1,1)
truth<-trueEIF.markov(o)
lambdavec<-c(1,1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10);

redv<-NULL;
for(i in 1:length(lambdavec)){
	redv[i]<-computederivative(lambdavec[i],o);
}
sredv<-round(signif(abs(redv-truth)/truth*100,2),1)

### Producing Figure 7 from output above

c<-NULL
for(i in 1:length(sredv)){

	if(sredv[i]==0) c[i]="forestgreen"
	if(sredv[i]>0&&sredv[i]<=1) c[i]="greenyellow"
	if(sredv[i]>1&&sredv[i]<=10) c[i]="gold"
	if(sredv[i]>10&&sredv[i]<=20) c[i]="orange1"
	if(sredv[i]>20) c[i]="red"
	
}

l<-0:10;
par(xpd=TRUE)
par(mar=c(8,0,4,5)+0.1)
plot(5,5,type='n',xlim=c(-0.5,11),ylim=c(0,10),xaxt='n',yaxt='n',axes=0,xlab="",ylab="")
#text(x=l,y=rep(1,11),labels=sredv,col=c)
points(x=l,y=rep(1,11),pch=15,col=c,cex=2.7)
labx=parse(text=paste(10, "^ ",paste("-",seq(0,10,by=1),sep=""), sep=""))
axis(1,at=0:10,labels=labx,xlab=2)
title(xlab=expression(lambda),cex.lab=1.5)

legend(10.75,2,legend=c("<0.1%","0.1-1%","1-10%","10-20%",">20%"),fill=c("forestgreen","greenyellow","gold","orange1","red"),bty="n")



